@@ -579,9 +579,10 @@ NTip.matrix <- function (phy) {
 #'
 #' `NSplits()` counts the unique bipartition splits in a tree or object.
 #'
-#' @param x A phylogenetic tree of class `phylo`, or a list of such trees
-#' (of class `list` or `multiPhylo`), or a `Splits` object,
-#' or a vector of integers.
+#' @param x A phylogenetic tree of class `phylo`; a list of such trees
+#' (of class `list` or `multiPhylo`); a `Splits` object;
+#' a vector of integers; or a character vector listing tips of a tree,
+#' or a character of length one specifying a tree in Newick format.
 #'
 #' @return `NSplits()` returns an integer specifying the number of bipartitions in
 #' the specified objects, or in a binary tree with `x` tips.
@@ -608,11 +609,11 @@ NSplits.phylo <- function (x) collapse.singles(x)$Nnode - 1L - TreeIsRooted(x)
 
 #' @rdname NSplits
 #' @export
-NSplits.multiPhylo <- function (x) vapply(x, NSplits, numeric(1L))
+NSplits.list <- function (x) vapply(x, NSplits, numeric(1L))
 
 #' @rdname NSplits
 #' @export
-NSplits.list <- NSplits.multiPhylo
+NSplits.multiPhylo <- NSplits.list
 
 #' @rdname NSplits
 #' @export
@@ -620,7 +621,19 @@ NSplits.Splits <- function (x) nrow(x)
 
 #' @rdname NSplits
 #' @export
-NSplits.numeric <- function (x) x - 3L
+NSplits.numeric <- function (x) pmax(0L, x - 3L)
+
+#' @rdname NSplits
+#' @importFrom ape read.tree
+#' @export
+NSplits.character <- function (x) {
+  lengthX <- length(x)
+  if (lengthX == 1) {
+    tree <- read.tree(text = x)
+    if (!is.null(tree)) return(NSplits(tree))
+  }
+  NSplits(lengthX)
+}
 
 #' Maximum splits in an _n_-leaf tree
 #'
